///////////////////////////////////////////////////////////////////////////////////////
// File     : api9014.c
// Function : API for ORLED Model 9014 controller with RL-28 ringlight
// Target   : OS independent, except for Sleep()
///////////////////////////////////////////////////////////////////////////////////////


#include <windows.h>		// for Sleep()

#include "include9014.h"


/////////////////////////////////////////////////////////////////////////////
// Receive a ringlight notification message (INFO + DATA bytes) in buf.
// Entry:
//	buf  - address of 2-byte buffer that will receive corrected INFO/DATA bytes.
//  wait - boolean: 1=wait for message, 0=return immediately if complete message has not been
//         received (but don't clear message buffer upon entry or exit).
// Returns:
//	0 on fail (timeout), 1 on success.

static int FetchReply(char *msgbuf, int wait)
{
	char c;
	int tStart;
	static char buf[2];
	static int nchars = 0;

	if (wait)
	{
		nchars = 0;
		tStart = GetTickCount();	 // Log starting time (msec since system startup - wraps after 2^32 msec).
	}

	while (1)		// Wait for reply or timeout
	{
		if (ComportRecv(&c))
		{
			if (c != 0)
			{
				if (nchars < 2)
					buf[nchars] = c;
				nchars++;
			}
			else if (nchars == 2)
			{
				if ((buf[0] & 0x80) != 0)		// If Z bit set in INFO byte,
				{
					msgbuf[0] = buf[0] & 0x7F;	//   Reset Z bit so only opcode remains.
					msgbuf[1] = 0;				//   Replace DATA value with 0x00.
				}
				else							// Otherwise just copy INFO and DATA.
				{
					msgbuf[0] = buf[0];
					msgbuf[1] = buf[1];
				}

				nchars = 0;		// Reset to receive next command.

				return 1;
			}
			else
				nchars = 0;		// Unexpected END byte -- reset to receive next command.
		}

		else if (!wait)
			return 0;

		else if ( (GetTickCount() - tStart) > 200)
			return 0;

		else
			Sleep(0);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////
// Public API Function: Receive and parse a notification message.
// Entry:
//	msg  - address of NMSG struct that will receive notification message.
//  wait - boolean: 1=wait for message, 0=return immediately if complete message has not been received.
//         If waiting, the recv msg buf will be cleared before starting, otherwise any partially
//         received message will be resumed. This is typically set to 1; it should be set to zero only
//         if asynchronous notification messages are enabled on the ringlight controller (via ASYNC command).
// Returns one of these:
//  MCLASS_NONE    - no message (timeout, if wait asserted)
//  MCLASS_SYNC    - command ack/nak
//  MCLASS_ASYNC   - async message
//  MCLASS_INVALID - unknown message type
// Notes:
//  1. Messages marked "*" are for intended for internal Orled use only.

MCLASS RinglightRecvMessage(NMSG *msg, int wait)
{
	// Wait for a complete received message. Exit if one has not been received.
	if (!FetchReply(msg->buf, wait))
		return MCLASS_NONE;

	// Extract message type and status information (including "pqqqqiii").
	msg->messageType	= msg->buf[0] & 0x1F;
	msg->powerFault		= (msg->buf[0] >> 6) & 1;
	msg->keyboardFault	= (msg->buf[0] >> 5) & 1;
	msg->powerEnable	= (msg->buf[1] >> 7) & 1;
	msg->quadEnable[3]	= (msg->buf[1] >> 6) & 1;
	msg->quadEnable[2]	= (msg->buf[1] >> 5) & 1;
	msg->quadEnable[1]	= (msg->buf[1] >> 4) & 1;
	msg->quadEnable[0]	= (msg->buf[1] >> 3) & 1;
	msg->intensity		= (msg->buf[1] & 7) + 1;

	// Decode and parse the message.
	switch(msg->messageType)
	{
	case MTYPE_CMDACK:		// Command ACK
	case MTYPE_CMDNAK:		// Command NAK (unrecognized command)
		return MCLASS_SYNC;

	case MTYPE_KEYUP:		// IntensityUp keypress
	case MTYPE_KEYDOWN:		// IntensityDown keypress
	case MTYPE_KEYPOWER:	// Power keypress
	case MTYPE_KEYQUAD0:	// Quadrant 0 keypress
	case MTYPE_KEYQUAD1:	// Quadrant 1 keypress
	case MTYPE_KEYQUAD2:	// Quadrant 2 keypress
	case MTYPE_KEYQUAD3:	// Quadrant 3 keypress
	case MTYPE_FAULT:		// Power/keyboard fault status change
		return MCLASS_ASYNC;

	default:
		msg->messageType = MTYPE_INVALID;	// Unknown message type.
		return MCLASS_INVALID;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////
// Public API Function: Construct a command message and send it to the ringlight controller.
// The controller will execute the command and return either an ACK or NAK command reply.
// It's not necessary to wait for the reply before issuing another command unless the
// client needs to verify ACK before proceeding.

void RinglightSendCommand(char opcode, char data)
{
	char buf[3];

	buf[0] = (data == 0) ? (opcode | 0x80) : opcode;	// Populate message buffer: Opcode and Z bit.
	buf[1] = (data == 0) ? 0xFF : data;					//   Data value, or 0xFF if value is zero.
	buf[2] = CMD_END;									//   Command message delimiter (END).

	ComportSend(buf, 3);								// Send message.
}
