///////////////////////////////////////////////////////////////////////////////////////
// File     : demo9014.c
// Function : Demo program for ORLED Model 9014 controller with RL-28 ringlight
// Target   : Windows console application
///////////////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////  OS-DEPENDENT CODE  ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <conio.h>
#include <stdio.h>

#include "include9014.h"


#define COM_PORTNUM			"COM1"		// Select serial port for ringlight control.


/////////////////////////////////////////////////////////////////////////////////////////
// Send command message to ringlight, then wait for reply and report ringlight status.
// NOTE: This assumes the ringlight controller is not operating in asynchronous mode.

static void ExecuteCommand(char opcode, char data)
{
	char	*ack;
	NMSG	nmsg;

	RinglightSendCommand(opcode, data);
	RinglightRecvMessage(&nmsg, 1);

	switch (nmsg.messageType)		// decode message type
	{
	case MTYPE_CMDACK:	ack = "CMDACK"; break;	// Command ACK
	case MTYPE_CMDNAK:	ack = "CMDNAK"; break;	// Command NAK
	default:
		printf("NO REPLY");
		return;
	}

	// Display ack/nak and ringlight status info.
	printf("%s pwr=%d quads=%d%d%d%d lvl=%d pf=%d kf=%d",
		ack,				// ack/nak type
		nmsg.powerEnable,
		nmsg.quadEnable[3],
		nmsg.quadEnable[2],
		nmsg.quadEnable[1],
		nmsg.quadEnable[0],
		nmsg.intensity,
		nmsg.powerFault,
		nmsg.keyboardFault);
}

/////////////////////////////////////////////////
// Execute ringlight control command.

static void CmdControl( int power, int quads, int intensity )
{
	if (intensity < 1)		intensity = 0;
	else if (intensity > 8)	intensity = 7;
	else					--intensity;

	ExecuteCommand( CMD_CONTROL, (char)( ((power & 1) << 7) | ((quads & 0xF) << 3) | intensity ) );
}

/////////////////////////////////////////////////////////////////////
// Demo - Send a sequence of commands to the ringlight controller.

#define DELAY	180		// milliseconds
#define NLOOPS	32

static void DemoControl( int power, int quadrants, int intensity, int delay )
{
	CmdControl(1, quadrants, intensity);
	printf("\n");
	Sleep(delay);
}

static void Rotate(int *quads, int intensity, int delay)
{
	DemoControl(1, *quads, intensity, delay);
	*quads = ((*quads << 1) + (*quads >> 3)) & 15;
}

static void Demo(void)
{
	int i, j;
	int quads;

	DemoControl(0, 1, 1, DELAY);

	quads = 1;

	// Rotate 1 quadrant
	for (i=0; i<NLOOPS; i++)
		Rotate(&quads, (8 * (i + 1))/NLOOPS, (DELAY * (NLOOPS - (i + 1))) / NLOOPS);
	for (j=8; j>0; j--)
		for (i=0; i<8; i++)
			Rotate(&quads, j, 0);
	for (i=0; i<NLOOPS; i++)
		Rotate(&quads, 1, 0);

	for (i=1; i<=8; i++)	// Ramp up intensity
		DemoControl(1, 15, i, DELAY);
	Sleep(1000);
	for (i=8; i>0; i--)	// Ramp down intensity
		DemoControl(1, 15, i, DELAY);

	CmdControl(0, 15, 1);
}

///////////////////////////////////////////////////////////////
// Display command menu.

void ShowMenu(void)
{
	// Show menu
	printf("COMMAND MENU -------\n");
	printf("w p q i - pwr, quads, intensity (e.g., 'w 1 1101 5')\n");
	printf("r       - read status\n");
	printf("d       - demo sequence\n");
	printf("q       - exit program\n");
	printf("?       - show this menu\n");
}

/////////////////////////////////////////////////////////////////
// Main function. Input and execute commands until user quits.

int main(void)
{
	int  rtnval;
	char buf[82];
	char *string;
	char cmd;
	char quad[80];
	int  intensity;
	int  data;
	int  nFields;

	printf("\nORLED 9014 CONTROLLER\n");

	rtnval = ComportOpen(COM_PORTNUM);			// open serial port

	if (rtnval == 0)
	{
		ShowMenu();

		do
		{
			// Input and parse command.
			printf("\n>");	// prompt
			buf[0] = sizeof(buf) - 2;
			string = _cgets(buf);
			nFields = sscanf(string, "%s %d %s %d", &cmd, &data, &quad, &intensity);

			if (nFields == EOF)
				continue;

			switch(cmd)		// Execute command.
			{
			case 'w':
				if ((nFields == 4))
					CmdControl( data, ((quad[0]&1)<<3) | ((quad[1]&1)<<2) | ((quad[2]&1)<<1) | (quad[3]&1), intensity );
				break;
			case 'r':
				ExecuteCommand(CMD_QUERY, 1);	
				break;
			case 'd':
				Demo();
				break;
			case '?':
				ShowMenu();
				break;
			case 'q':
				break;
			default:
				printf("BADCMD");
				break;
			}
		}
		while (cmd != 'q');
	}

	ComportClose();

	return rtnval;
}
